@@ -16,60 +16,52 @@ const char *ripe_servers[] = {
     "whois.aunic.net",
     "whois.connect.com.au",	/* 3.0.0b1 */
     "whois.nic.fr",
+    "whois.nic.it",
+    "whois.cw.net",
     "whois.telstra.net",
     "whois.nic.net.sg",
     "whois.metu.edu.tr",
     "whois.restena.lu",
     "rr.level3.net",		/* 3.0.0a13 */
+    "whois.ripn.net",
     "whois.arnes.si",
     "www.registry.co.ug",
     "whois.nic.ir",
+    "whois.nic.ck",
     NULL
 };
 
 /* servers which do not accept the new syntax */
 const char *ripe_servers_old[] = {
     "whois.ra.net",
-    "whois.nic.it",
-    "whois.ans.net",
-    "whois.cw.net",
-    "whois.ripn.net",
-    "whois.nic.ck",
     "whois.domain.kg",
+    "whois.nic.ch",
     NULL
 };
 
-#if 0
-const char *rwhois_servers[] = {
-    "whois.isi.edu",		/* V-1.0B9.2 */
-    "rwhois.rcp.net.pe",	/* V-1.5.3 */
-    "ns.twnic.net",		/* V-1.0B9 */
-    "dragon.seed.net.tw",	/* V-1.0B9.2 */
-    NULL
-};
-#endif
-
 const char *hide_strings[] = {
-    "The Data in the VeriSign", "terms at any time.",
-    "The data in Register", "By submitting",
-    " The data contained in Dotster", "Please limit your",
+    "The Data in the VeriSign", "terms at any time.",		/* VERISIGN */
+    "The data in Register", "By submitting",		    /* REGISTER.COM */
+    "The Data in the Tucows", "RECORD DOES NOT",		/* OPENSRS */
+    " The data contained in Dotster", "Please limit your",	/* DOTSTER */
     "This whois service currently only", "top-level domains.",
     "Signature Domains' Whois Service", "agree to abide by the above",
-    "Access to ASNIC", "by this policy.",
-    "* Copyright (C) 1998 by SGNIC", "* modification.",
+    "Access to ASNIC", "by this policy.",			/* as */
+    "**************", "**************",				/* sg */
     "The Data in Gabia", "you agree to abide",
-    "NeuLevel, Inc., the Registry Operator", "whatsoever, you agree",
-    "NOTICE: Access to .INFO WHOIS", "time. By submitting",
+    "The data contained in Go Daddy", "is not the owner",	/* NEUSTAR */
+    "NOTICE: Access to .INFO WHOIS", "time. By submitting",	/* info */
     "Disclaimer: The Global Name Registry", "for any commercial",
-    "Access to America Online", "time. By accessing",
-    "Access and use restricted", "http://www.icann", /* GANDI */
+    "Access to America Online", "time. By accessing",		/* AOL */
+    "% Access and use restricted", "% http://www.icann",	/* GANDI */
+    "NeuStar, Inc., the Registry", "rules.  For details",	/* us */
+    "The data in this whois", "Version 6.3",			/* enom */
     NULL, NULL
 };
 
 const char *nic_handles[] = {
     "net-",	"whois.arin.net",
     "netblk-",	"whois.arin.net",
-    "asn-",	"whois.arin.net",
     "lim-",	"whois.ripe.net",
     "coco-",	"whois.corenic.net",
     "coho-",	"whois.corenic.net",
@@ -95,6 +87,23 @@ struct ip_del ip_assign[] = {
     { 0, 0, NULL }
 };
 
+struct ip6_del {
+    unsigned long net;		/* bits 16-22 of the address */
+    const char    *serv;
+};
+
+struct ip6_del ip6_assign[] = {
+    { 0x0200, "whois.apnic.net" },
+    { 0x0400, "whois.arin.net" },
+    { 0x0600, "whois.ripe.net" },
+    { 0x0800, "whois.ripe.net" },
+    { 0x0A00, "whois.ripe.net" },
+    { 0x0C00, "whois.apnic.net" },
+    { 0x0E00, "whois.apnic.net" },
+    { 0x1200, "whois.lacnic.net" },
+    { 0, NULL }
+};
+
 struct as_del {
     unsigned short first;
     unsigned short last;
