@@ -26,8 +26,6 @@
 #include <errno.h>
 #include <signal.h>
 
-#undef ENABLE_NLS
-
 /* Application-specific */
 #include "data.h"
 #include "whois.h"
@@ -41,11 +39,13 @@ int hide_discl = 0;
 int hide_discl = 2;
 #endif
 
+char *client_tag = (char *)IDSTRING;
+
 #ifdef HAVE_GETOPT_LONG
 static struct option longopts[] = {
     {"help",	no_argument,		NULL, 0  },
     {"version",	no_argument,		NULL, 1  },
-    {"verbose",	no_argument,		NULL, 'V'},
+    {"verbose",	no_argument,		NULL, 2  },
     {"server",	required_argument,	NULL, 'h'},
     {"host",	required_argument,	NULL, 'h'},
     {"port",	required_argument,	NULL, 'p'},
@@ -68,7 +68,7 @@ int main(int argc, char *argv[])
     textdomain(NLS_CAT_NAME);
 #endif
 
-    while ((ch = GETOPT_LONGISH(argc, argv, "acFg:h:Hi:lLmMp:q:rRs:St:T:v:Vx",
+    while ((ch = GETOPT_LONGISH(argc, argv, "acdFg:h:Hi:KlLmMp:q:rRs:St:T:v:V:x",
 				longopts, 0)) > 0) {
 	/* RIPE flags */
 	if (strchr(ripeflags, ch)) {
@@ -92,13 +92,15 @@ int main(int argc, char *argv[])
 		port = p + 1;
 	    *q = '\0';
 	    break;
+	case 'V':
+	    client_tag = optarg;
 	case 'H':
 	    hide_discl = 0;	/* enable disclaimers hiding */
 	    break;
 	case 'p':
 	    port = optarg;
 	    break;
-	case 'V':
+	case 2:
 	    verb = 1;
 	    break;
 	case 1:
@@ -120,7 +122,7 @@ int main(int argc, char *argv[])
 	usage();
 
     /* On some systems realloc only works on non-NULL buffers */
-    qstring = malloc(1);
+    qstring = malloc(64);
     *qstring = '\0';
 
     /* parse other parameters, if any */
@@ -150,11 +152,18 @@ int main(int argc, char *argv[])
     }
 #endif
 
+    signal(SIGTERM, sighandler);
+    signal(SIGINT, sighandler);
+    signal(SIGALRM, alarm_handler);
+    alarm(60);
+
     if (!server) {
 	char *tmp;
+
 	tmp = normalize_domain(qstring);
 	server = whichwhois(tmp);
 	free(tmp);
+
 	switch (server[0]) {
 	    case 0:
 		if (!(server = getenv("WHOIS_SERVER")))
@@ -173,14 +182,21 @@ int main(int argc, char *argv[])
 		exit(0);
 	    case 4:
 		if (verb)
-		    puts(_("Connecting to whois.internic.net."));
-		sockfd = openconn("whois.internic.net", NULL);
+		    puts(_("Connecting to whois.crsnic.net."));
+		sockfd = openconn("whois.crsnic.net", NULL);
 		server = query_crsnic(sockfd, qstring);
-		closeconn(sockfd);
+		close(sockfd);
 		if (!server)
 		    exit(0);
-		printf(_("\nFound InterNIC referral to %s.\n\n"), server);
+		printf(_("\nFound crsnic referral to %s.\n\n"), server);
+		alarm(60);
 		break;
+	    case 5:
+		puts(_("No whois server is known for this kind of object."));
+		exit(0);
+	    case 6:
+		puts(_("Unknown AS number or IP network. Please upgrade this program."));
+		exit(0);
 	    default:
 		if (verb)
 		    printf(_("Using server %s.\n"), server);
@@ -195,10 +211,13 @@ int main(int argc, char *argv[])
 	printf(_("Query string: \"%s\"\n\n"), p);
     strcat(p, "\r\n");
 
-    signal(SIGTERM, sighandler);
-    signal(SIGINT, sighandler);
-
     sockfd = openconn(server, port);
+
+    /*
+     * Now we are connected and the query is supposed to complete quickly.
+     * This will help people who run whois ... | less
+     */
+    alarm(0);
     do_query(sockfd, p);
 
     exit(0);
@@ -282,22 +301,25 @@ const char *whichwhois(const char *s)
 
     /* IPv6 address */
     if (strchr(s, ':')) {
-	if (strncasecmp(s, "2001:2", 6) == 0 ||	/* XXX ugly hack! */
-	    strncasecmp(s, "2001:02", 6) == 0)
-	    return "whois.apnic.net";
-	if (strncasecmp(s, "2001:4", 6) == 0 ||
-	    strncasecmp(s, "2001:04", 6) == 0)
-	    return "whois.arin.net";
-	if (strncasecmp(s, "2001:6", 6) == 0 ||
-	    strncasecmp(s, "2001:06", 6) == 0)
-	    return "whois.ripe.net";
-	/* if (strncasecmp(s, "3ffe", 4) == 0) */
+	if (strncmp(s, "2001:",  5) == 0) {
+	    unsigned long v6net = strtol(s + 5, NULL, 16);
+	    v6net = v6net & 0xfe00;	/* we care about the first 7 bits */
+	    for (i = 0; ip6_assign[i].serv; i++)
+		if (v6net == ip6_assign[i].net)
+		    return ip6_assign[i].serv;
+	    return "\006";			/* unknown allocation */
+	} else if (strncasecmp(s, "3ffe:", 5) == 0)
 	    return "whois.6bone.net";
+	/* RPSL objects like AS8627:fltr-TRANSIT-OUT */
+	else if (strncasecmp(s, "as", 2) == 0 && isasciidigit(s[2]))
+	    return whereas(atoi(s + 2));
+	else
+	    return "\005";
     }
 
     /* email address */
     if (strchr(s, '@'))
-	return "";
+	return "\005";
 
     /* no dot and no hyphen means it's a NSI NIC handle or ASN (?) */
     if (!strpbrk(s, ".-")) {
@@ -305,14 +327,14 @@ const char *whichwhois(const char *s)
 
 	for (p = s; *p; p++);			/* go to the end of s */
 	if (strncasecmp(s, "as", 2) == 0 &&	/* it's an AS */
-	    ((s[2] >= '0' && s[2] <= '9') || s[2] == ' '))
-	    return whereas(atoi(s + 2), as_assign);
+		(isasciidigit(s[2]) || s[2] == ' '))
+	    return whereas(atoi(s + 2));
 	else if (strncasecmp(p - 2, "jp", 2) == 0) /* JP NIC handle */
 	    return "whois.nic.ad.jp";
-	if (*p == '!')	/* NSI NIC handle */
+	if (*s == '!')	/* NSI NIC handle */
 	    return "whois.networksolutions.com";
-	else /* it's a NSI NIC handle or something we don't know about */
-	    return "";
+	else
+	    return "\005";	/* probably a unknown kind of nic handle */
     }
 
     /* smells like an IP? */
@@ -320,13 +342,10 @@ const char *whichwhois(const char *s)
 	for (i = 0; ip_assign[i].serv; i++)
 	    if ((ip & ip_assign[i].mask) == ip_assign[i].net)
 		return ip_assign[i].serv;
-	if (verb)
-	    puts(_("I don't know where this IP has been delegated.\n"
-		   "I'll try ARIN and hope for the best..."));
-	return "whois.arin.net";
+	return "\005";			/* not in the unicast IPv4 space */
     }
 
-    /* check TLD list */
+    /* check the TLDs list */
     for (i = 0; tld_serv[i]; i += 2)
 	if (domcmp(s, tld_serv[i]))
 	    return tld_serv[i + 1];
@@ -337,29 +356,23 @@ const char *whichwhois(const char *s)
 	for (i = 0; nic_handles[i]; i += 2)
 	    if (strncasecmp(s, nic_handles[i], strlen(nic_handles[i])) == 0)
 		return nic_handles[i + 1];
-	if (verb)
-	    puts(_("I guess it's a netblock name but I don't know where to"
-		   " look it up."));
-	return "whois.arin.net";
+	/* it's probably a network name */
+	return "";
     }
 
-    /* has dot and hypen and it's not in tld_serv[], WTF is it? */
-    if (verb)
-	puts(_("I guess it's a domain but I don't know where to look it"
-	       " up."));
-    return "";
+    /* has dot and maybe a hypen and it's not in tld_serv[], WTF is it? */
+    /* either a TLD or a NIC handle we don't know about yet */
+    return "\005";
 }
 
-const char *whereas(int asn, struct as_del aslist[])
+const char *whereas(const unsigned short asn)
 {
     int i;
 
-    if (asn > 23551)
-	puts(_("Unknown AS number. Please upgrade this program."));
-    for (i = 0; aslist[i].serv; i++)
-	if (asn >= aslist[i].first && asn <= aslist[i].last)
-	    return aslist[i].serv;
-    return "whois.arin.net";
+    for (i = 0; as_assign[i].serv; i++)
+	if (asn >= as_assign[i].first && asn <= as_assign[i].last)
+	    return as_assign[i].serv;
+    return "\006";
 }
 
 char *queryformat(const char *server, const char *flags, const char *query)
@@ -368,38 +381,39 @@ char *queryformat(const char *server, const char *flags, const char *query)
     int i, isripe = 0;
 
     /* +10 for CORE; +2 for \r\n; +1 for NULL */
-    buf = malloc(strlen(flags) + strlen(query) + 10 + 2 + 1);
+    buf = malloc(strlen(flags) + strlen(query) + strlen(client_tag) + 4
+	    + 10 + 2 + 1);
     *buf = '\0';
     for (i = 0; ripe_servers[i]; i++)
 	if (strcmp(server, ripe_servers[i]) == 0) {
-	    strcat(buf, "-V " IDSTRING " ");
+	    strcat(buf, "-V ");
+	    strcat(buf, client_tag);
+	    strcat(buf, " ");
 	    isripe = 1;
 	    break;
 	}
     if (!isripe)
 	for (i = 0; ripe_servers_old[i]; i++)
 	    if (strcmp(server, ripe_servers_old[i]) == 0) {
-		strcat(buf, "-V" IDSTRING " ");
+		strcat(buf, "-V");
+		strcat(buf, client_tag);
+		strcat(buf, " ");
 		isripe = 1;
 		break;
 	    }
     if (*flags) {
 	if (!isripe && strcmp(server, "whois.corenic.net") != 0)
-	    puts(_("Warning: RIPE flags ignored for a traditional server."));
-	else
-	    strcat(buf, flags);
+	    puts(_("Warning: RIPE flags used with a traditional server."));
+	strcat(buf, flags);
     }
-    if (!isripe &&
-	    (strcmp(server, "whois.arin.net") == 0 ||
-	     strcmp(server, "whois.nic.mil") == 0) &&
-	    strncasecmp(query, "AS", 2) == 0 &&
-	    query[2] >= '0' && query[2] <= '9')
-	sprintf(buf, "AS %s", query + 2);	/* fix query for ARIN */
+    if (!isripe && strcmp(server, "whois.nic.mil") == 0 &&
+	    strncasecmp(query, "AS", 2) == 0 && isasciidigit(query[2]))
+	sprintf(buf, "AS %s", query + 2);	/* fix query for DDN */
+    if (!isripe && strcmp(server, "whois.arin.net") == 0 &&
+	    strncasecmp(query, "AS", 2) == 0 && isasciidigit(query[2]))
+	sprintf(buf, "A %s", query + 2);	/* always ask for a ASN */
     else if (!isripe && strcmp(server, "whois.corenic.net") == 0)
 	sprintf(buf, "--machine %s", query);	/* machine readable output */
-    else if (!isripe && strcmp(server, "whois.ncst.ernet.in") == 0 &&
-	     !strchr(query, ' '))
-	sprintf(buf, "domain %s", query);	/* ask for a domain */
     else if (!isripe && strcmp(server, "whois.nic.ad.jp") == 0) {
 	char *lang = getenv("LANG");	/* not a perfect check, but... */
 	if (!lang || (strncmp(lang, "ja", 2) != 0))
@@ -512,8 +526,8 @@ int openconn(const char *server, const char *port)
 {
     int fd;
 #ifdef HAVE_GETADDRINFO
-    int i;
-    struct addrinfo hints, *res, *ressave;
+    int err;
+    struct addrinfo hints, *res, *ai;
 #else
     struct hostent *hostinfo;
     struct servent *servinfo;
@@ -525,18 +539,18 @@ int openconn(const char *server, const char *port)
     hints.ai_family = AF_UNSPEC;
     hints.ai_socktype = SOCK_STREAM;
 
-    if ((i = getaddrinfo(server, port ? port : "whois", &hints, &res)) != 0)
-	err_quit("getaddrinfo: %s", gai_strerror(i));
-    for (ressave = res; res; res = res->ai_next) {
-	if ((fd = socket(res->ai_family, res->ai_socktype, res->ai_protocol))<0)
+    if ((err = getaddrinfo(server, port ? port : "whois", &hints, &res)) != 0)
+	err_quit("getaddrinfo: %s", gai_strerror(err));
+    for (ai = res; ai; ai = ai->ai_next) {
+	if ((fd = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol)) < 0)
 	    continue;		/* ignore */
-	if (connect(fd, (struct sockaddr *)res->ai_addr, res->ai_addrlen) == 0)
+	if (connect(fd, (struct sockaddr *)ai->ai_addr, ai->ai_addrlen) == 0)
 	    break;		/* success */
 	close(fd);
     }
-    freeaddrinfo(ressave);
+    freeaddrinfo(res);
 
-    if (!res)
+    if (!ai)
 	err_sys("connect");
 #else
     if ((hostinfo = gethostbyname(server)) == NULL)
@@ -559,14 +573,15 @@ int openconn(const char *server, const char *port)
     return fd;
 }
 
-void closeconn(const int fd)
+void alarm_handler(int signum)
 {
-    close(fd);
+    close(sockfd);
+    err_quit(_("Timeout."));
 }
 
 void sighandler(int signum)
 {
-    closeconn(sockfd);
+    close(sockfd);
     err_quit(_("Interrupted by signal %d..."), signum);
 }
 
@@ -607,34 +622,40 @@ unsigned long myinet_aton(const char *s)
     return (a << 24) + (b << 16) + (c << 8) + d;
 }
 
+int isasciidigit(const char c) {
+    return (c >= '0' && c <= '9') ? 1 : 0;
+}
+
+/* http://www.ripe.net/ripe/docs/databaseref-manual.html */
+
 void usage(void)
 {
     fprintf(stderr, _(
 "Usage: whois [OPTION]... OBJECT...\n\n"
-"-a                     search all databases\n"
-"-F                     fast raw output (implies -r)\n"
-"-g SOURCE:FIRST-LAST   find updates from SOURCE from serial FIRST to LAST\n"
-"-h HOST                connect to server HOST\n"
-"-H                     hide legal disclaimers\n"
-"-i ATTR[,ATTR]...      do an inverse lookup for specified ATTRibutes\n"
-"-x                     exact match [RPSL only]\n"
 "-l                     one level less specific lookup [RPSL only]\n"
 "-L                     find all Less specific matches\n"
-"-M                     find all More specific matches\n"
 "-m                     find first level more specific matches\n"
-"-r                     turn off recursive lookups\n"
-"-p PORT                connect to PORT\n"
+"-M                     find all More specific matches\n"
+"-c                     find the smallest match containing a mnt-irt attribute\n"
+"-x                     exact match [RPSL only]\n"
+"-d                     return DNS reverse delegation objects too [RPSL only]\n"
+"-i ATTR[,ATTR]...      do an inverse lookup for specified ATTRibutes\n"
+"-T TYPE[,TYPE]...      only look for objects of TYPE\n"
+"-K                     only primary keys are returned [RPSL only]\n"
+"-r                     turn off recursive lookups for contact information\n"
 "-R                     force to show local copy of the domain object even\n"
 "                       if it contains referral\n"
-"-S                     tell server to leave out syntactic sugar\n"
+"-a                     search all databases\n"
 "-s SOURCE[,SOURCE]...  search the database from SOURCE\n"
-"-T TYPE[,TYPE]...      only look for objects of TYPE\n"
+"-g SOURCE:FIRST-LAST   find updates from SOURCE from serial FIRST to LAST\n"
 "-t TYPE                request template for object of TYPE ('all' for a list)\n"
 "-v TYPE                request verbose template for object of TYPE\n"
-"-q [version|sources]   query specified server info [RPSL only]\n"
-"-d                     return DNS reverse delegation objects too [RPSL only]\n"
-"-K                     only primary keys are returned [RPSL only]\n"
-"-V    --verbose        explain what is being done\n"
+"-q [version|sources|types]  query specified server info [RPSL only]\n"
+"-F                     fast raw output (implies -r)\n"
+"-h HOST                connect to server HOST\n"
+"-p PORT                connect to PORT\n"
+"-H                     hide legal disclaimers\n"
+"      --verbose        explain what is being done\n"
 "      --help           display this help and exit\n"
 "      --version        output version information and exit\n"
 ));
